# dr14_t.meter: compute the DR14 value of the given audiofiles
# Copyright (C) 2011  Simone Riva
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import numpy
from dr14tmeter.audio_math import *

try:
    import matplotlib.pyplot as pyplot
    import matplotlib.mlab as mlab
    import pylab
except:
    ____foo = None
 
    
def spectrogram( Y , Fs ):
    
    s = Y.shape
    
    if len( Y.shape ) > 1 :
        ch = s[1]
    else :
        ch = 1
    
    #Ym = numpy.sum( Y , 1 ) / float(ch)
    
    
    for j in numpy.arange(ch):
        pyplot.subplot(210+j+1)
        pylab.specgram( Y[:,j] , NFFT=Fs , Fs=Fs , cmap='gnuplot2' )
        
        mean_x = numpy.array([ 0 , s[0]*(1/Fs) ])
        mean_x = numpy.array([ mean , mean ])
        
        pyplot.axis([ 0, s[0]*(1/Fs), 0, Fs / 2.0 ])
    
        pyplot.title( "Channel %d" % j )
        pyplot.xlabel('Time [sec]')
        pyplot.ylabel('Freq. [Hz]')
        pyplot.grid(True)
        
        pyplot.title( "Channel: %d"%int(j+1))
     
    pyplot.show()


# dr14_t.meter: compute the DR14 value of the given audiofiles
# Copyright (C) 2011  Simone Riva
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from numpy  import *
from dr14tmeter.audio_math import *
import math
import numpy

def compute_DRV( Y , Fs , duration = None , Dr_lr = None ) :
    
    s = Y.shape
    
    if len( Y.shape ) > 1 :
        ch = s[1]
    else :
        ch = 1
    
    block_time = 1
    block_samples = block_time * ( Fs )
    
    threshold = 0.15    
    seg_cnt = int( floor( s[0] / block_samples ) )
    
    if seg_cnt < 3 :
        return ( 0 , -100 , -100 )

    curr_sam = 0   
    
    rms = zeros((seg_cnt,ch))
    peaks = zeros((seg_cnt,ch))    
    
    for i in range( seg_cnt - 1) :
        r = arange( curr_sam , curr_sam + block_samples )
        
        rms[i,:] = decibel_u( u_rms( Y[r,:] ) , 1.0 )
        peaks[i,:] = decibel_u( numpy.max( numpy.abs( Y[r,:] ) ) , 1.0 )
        
        curr_sam = curr_sam + block_samples
    
    i = numpy.logical_or( rms == numpy.nan , rms == -numpy.inf ) 
    rms[i] = decibel_u( audio_min() , 1.0 )

    i = numpy.logical_or( peaks == numpy.nan , peaks == -numpy.inf ) 
    peaks[i] = decibel_u( audio_min() , 1.0 )   
    
    Ydr = numpy.mean( peaks - rms , 1 )
    
    (n,bins) = numpy.histogram( Ydr , 100 )
    
    #print( peaks )
    #print( rms )
    
    max_freq = numpy.max( n )
       
    bs = bins.shape[0]
    bins = bins[0:bs-1] + numpy.diff( bins ) / 2.0
    
    i = n > max_freq*threshold
    
    n = n[i]
    bins = bins[i]
    
    m = numpy.sum( n * bins ) / numpy.sum( n )
    
    drV = round( m - 3 )   
    
    dB_peak = numpy.max( peaks )
    
    dB_rms = numpy.sum( numpy.mean( rms , 0 ) ) / 2.0 
    
    if duration != None :
        duration.tm_min = int( s[0] * (1.0 / Fs) / 60.0 )
        duration.tm_sec = int( (( s[0] * (1.0 / Fs) / 60.0 ) - duration.tm_min ) * 60.0 )
        
    if Dr_lr != None :
        Dr_lr = ch_dr14    
    
    return ( drV , dB_peak , dB_rms )



# dr14_t.meter: compute the DR14 value of the given audiofiles
# Copyright (C) 2011  Simone Riva
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import os
import sys


class Table:
    
    def nl(self):
        if sys.platform.startswith('linux') or sys.platform.startswith('darwin'):
            return '\n'
        elif sys.platform.startswith('win'):
            return '\n\r'
    
    def append_row( self , txt , row_el , cell_type='d'):
        
        if cell_type == 'd':
            n_cell = self.new_cell
            e_cell = self.end_cell
        elif cell_type == 'h':
            n_cell = self.new_hcell
            e_cell = self.end_hcell
        
        txt = self.new_row(txt)
        
        for i in row_el:
            txt = n_cell(txt)
            txt = txt + i
            txt = e_cell(txt)
            
        txt = self.end_row(txt)
        return txt

    def col_cnt( self ):
        return 5

    def append_separator_line( self , txt ):
        return txt
    
    def append_closing_line( self , txt ):
        return txt
        
    def append_empty_line( self , txt ):
        return self.append_row( txt , [ "", "", "", "", "" ] )

    def add_title( self , txt , title ):
        NotImplementedError(" %s : is virutal and must be overridden." % sys._getframe().f_code.co_name )
    
    def new_table( self , txt ):
        NotImplementedError(" %s : is virutal and must be overridden." % sys._getframe().f_code.co_name )
    
    def end_table( self , txt ):
        NotImplementedError(" %s : is virutal and must be overridden." % sys._getframe().f_code.co_name )
    
    def new_head( self , txt ):
        return txt
    
    def end_head( self , txt ):
        return txt
    
    def new_tbody( self , txt ):
        return txt
    
    def end_tbody( self , txt ):
        return txt
    
    def new_foot( self , txt ):
        return txt
    
    def end_foot( self , txt ):
        return txt
    
    def new_row( self , txt ):
        NotImplementedError(" %s : is virutal and must be overridden." % sys._getframe().f_code.co_name )
    
    def end_row( self , txt ):
        NotImplementedError(" %s : is virutal and must be overridden." % sys._getframe().f_code.co_name )
    
    def new_cell( self , txt ):
        NotImplementedError(" %s : is virutal and must be overridden." % sys._getframe().f_code.co_name )
    
    def end_cell( self , txt ):
        NotImplementedError(" %s : is virutal and must be overridden." % sys._getframe().f_code.co_name )
    
    def new_hcell( self , txt ):
        return self.new_cell( txt )
    
    def end_hcell( self , txt ):
        return self.end_cell( txt )
    
    def new_bold( self , txt ):
        NotImplementedError(" %s : is virutal and must be overridden." % sys._getframe().f_code.co_name )
    
    def end_bold( self , txt ):
        NotImplementedError(" %s : is virutal and must be overridden." % sys._getframe().f_code.co_name )
    
    
    

class TextTable ( Table ):

    def append_separator_line( self , txt ):
        return self.append_row( txt , [ "----------------------------------------------------------------------------------------------" ] )

    def append_closing_line( self , txt ):
        return self.append_row( txt , [ "==============================================================================================" ] )
    
    def append_empty_line( self , txt ):
        return self.append_row( txt , [ "" ] ) 

    def add_title( self , txt , title ):
        return txt + title + self.nl()

    def new_table( self , txt ):
        return txt
    
    def end_table( self , txt ):
        return txt + self.nl()
    
    def new_row( self , txt ):
        return txt + ''
    
    def end_row( self , txt ):
        return txt + self.nl()
    
    def new_cell( self , txt ):
        return txt + ''
    
    def end_cell( self , txt ):
        return txt + '\t'
    
    def new_bold( self , txt ):
        return txt + ''
    
    def end_bold( self , txt ):
        return txt + ''
    
    

class BBcodeTable ( Table ):

    def append_separator_line( self , txt ):
        return self.append_row( txt , [ "-----------", "-----------", "-----------", "-----------", "-------------------------------" ] )

    def append_closing_line( self , txt ):
        return self.append_row( txt , [ "===========", "===========", "===========", "===========", "===============================" ] )

    def add_title( self , txt , title ):
        return txt + self.nl() + "[tr]" + self.nl() + " [td  colspan=%d] " % self.col_cnt() + title + " [/td] " + self.nl() + "[/tr]" + self.nl()

    def new_table( self , txt ):
        return txt + '[table]' + self.nl()
    
    def end_table( self , txt ):
        return txt + self.nl() + '[/table]' + self.nl() 
    
    def new_row( self , txt ):
        return txt + self.nl() + '[tr]' + self.nl()
    
    def end_row( self , txt ):
        return txt + self.nl() + '[/tr]' + self.nl()
    
    def new_cell( self , txt ):
        return txt + ' [td]'
    
    def end_cell( self , txt ):
        return txt + '[/td]'
    
    def new_bold( self , txt ):
        return txt + '[b]'
    
    def end_bold( self , txt ):
        return txt + '[/b]'


class HtmlTable ( Table ):

    def add_title( self , txt , title ):
        return txt + self.nl() + "<tr>" + self.nl() + " <th colspan=\"%d\" > " % self.col_cnt() + title + "</th>" + self.nl() + "</tr>" + self.nl() 

    def new_table( self , txt ):
        return txt + "<table>" + self.nl() 
    
    def end_table( self , txt ):
        return txt + self.nl() + "</table>" + self.nl()
        
    def new_head( self , txt ):
        return txt + self.nl() + "<thead>" + self.nl() 
    
    def end_head( self , txt ):
        return txt + self.nl() + "</thead>" + self.nl()
        
    def new_tbody( self , txt ):
        return txt + self.nl() + "<tbody>" + self.nl() 
    
    def end_tbody( self , txt ):
        return txt + self.nl() + "</tbody>" + self.nl() 
    
    def new_foot( self , txt ):
        return txt + self.nl() + "<tfoot>" + self.nl() 
    
    def end_foot( self , txt ):
        return txt + self.nl() + "</tfoot>" + self.nl() 
    
    def new_row( self , txt ):
        return txt + self.nl() + "<tr>" + self.nl() 
    
    def end_row( self , txt ):
        return txt + self.nl() + "</tr>" + self.nl() 
    
    def new_cell( self , txt ):
        return txt + ' <td>'
    
    def end_cell( self , txt ):
        return txt + '</td>'
    
    def new_hcell( self , txt ):
        return txt + ' <th>'
    
    def end_hcell( self , txt ):
        return txt + '</th>'
    
    def new_bold( self , txt ):
        return txt + '<b>'
    
    def end_bold( self , txt ):
        return txt + '</b>'


class MediaWikiTable ( Table ):

    def add_title( self , txt , title ):
        return txt + "|-" + self.nl() + "!align=\"left\" colspan=\"%d\" | " % self.col_cnt() + title + self.nl()

    def new_table( self , txt ):
        return txt + "{| " + self.nl() 
    
    def end_table( self , txt ):
        return txt + "|}" + self.nl()
        
    def new_row( self , txt ):
        return txt + "|-" + self.nl() 
    
    def end_row( self , txt ):
        return txt + self.nl()
    
    def new_cell( self , txt ):
        return txt + '||'
    
    def end_cell( self , txt ):
        return txt
        
    def new_bold( self , txt ):
        return txt + "\'\'\'"
    
    def end_bold( self , txt ):
        return txt + "\'\'\'"




# dr14_t.meter: compute the DR14 value of the given audiofiles
# Copyright (C) 2011  Simone Riva
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import numpy
import time

from dr14tmeter.audio_math import *
from dr14tmeter.out_messages import *

try:
    from scipy.interpolate import interp1d
except:
    ____foo = None




class DynCompressor:
    
    def __init__(self):
        self.maxDB = -3.0
        self.linear_limit = 0.3

    def set_compression_modality( self , modality="medium" ):
        if modality == "very_soft" or modality == "v":
            self.maxDB = -2.0
            self.linear_limit = 0.5
        elif modality == "soft" or modality == "s" :
            self.maxDB = -3.0
            self.linear_limit = 0.3
        elif modality == "medium" or modality == "m" :
            self.maxDB = -5.0
            self.linear_limit = 0.3
        elif modality == "hard" or modality == "h" :
            self.maxDB = -7.0
            self.linear_limit = 0.3
        elif modality == "very_hard" or modality == "vh":
            self.maxDB = -9.0
            self.linear_limit = 0.2
        else:
            self.maxDB = -3.0
            self.linear_limit = 0.3
            
        print( "%f %f " % ( self.maxDB , self.linear_limit ) )


    def dyn_compressor( self , Y , Fs ):
        
        time_a = time.time()
        
        x = numpy.linspace( -1 , 1 , 21 )
        y = self.c_fun( x )
        
        int_f = interp1d( x , y , kind='cubic' )
           
        cY = int_f( Y )
        
        cY[:] = normalize( cY )
        
        time_b = time.time()
        dr14_log_info( "dyn_compressor: Clock: %2.8f" % (time_b - time_a ) )
            
        return cY
    
    
    def c_fun( self , x ):
        
        z = self.linear_limit
        
        y = numpy.zeros( x.shape )
        
        r = numpy.abs(x)<=(z+z*10e-6)
        y[r] = 0.0
        
        r2 = numpy.abs(x)>z
        
        a = -self.maxDB / (z-1.0)
        b = -a * z
        
        y[r2] =  (a * numpy.abs(x[r2]) + b )
        
        return x * 10.0**( y / 20.0 )
    



# dr14_t.meter: compute the DR14 value of the given audiofiles
# Copyright (C) 2011  Simone Riva
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import numpy
from dr14tmeter.audio_math import *

try:
    import matplotlib.pyplot as pyplot
    import matplotlib.mlab as mlab
except:
    ____foo = None


def compute_hist ( Y , Fs , duration=None , bins=100 , block_duration = 0.2 , plot=True , title=None ):
    s = Y.shape
    
    if len( Y.shape ) > 1 :
        ch = s[1]
    else :
        ch = 1

    saples_per_block = int( Fs * block_duration )
    
    seg_cnt = int( math.floor( float(s[0]) / saples_per_block ) ) + 1 
    
    if seg_cnt < 3:
        return False
    
    curr_sam = 0
    rms = zeros((seg_cnt,ch))
    
    #peaks = zeros((seg_cnt,ch))
    
    for i in range( seg_cnt - 1 ):
        #r = arange( curr_sam , curr_sam + saples_per_block )
        rms[i,:] = u_rms( Y[curr_sam : curr_sam + saples_per_block , : ] )
        curr_sam = curr_sam + saples_per_block
       
    i = seg_cnt - 1 ;
    r = arange( curr_sam,s[0] )
    
    if r.shape[0] > 0:
        rms[i,:] = dr_rms( Y[r,:] ) 
    
    rms = numpy.sum( rms , 1 ) / float(ch)
    
    rms[rms==0.0] = audio_min16()
    rms = decibel_u( rms , 1.0 )
    
    rms_mean = numpy.mean( rms )
    rms_std = numpy.std( rms )
    rms_max = numpy.max( rms )
    
    abs_peak = decibel_u( numpy.max( numpy.abs( Y ) ) , 1.0 )
    
    if plot == True :
        ( hist , bin_edges , patches ) = pyplot.hist( rms , 100 , normed=True )
        
        mean_x = numpy.array([ rms_mean , rms_mean ])
        mean_y = numpy.array([ 0.0 , numpy.max(hist)*1.01 ])
        
        std_x = numpy.array([ rms_mean - rms_std , rms_mean + rms_std ])
        std_y = numpy.array([ numpy.max(hist)*0.7 , numpy.max(hist)*0.7 ])
        
        pyplot.plot( mean_x , mean_y , linewidth=2 , color='g')
        pyplot.plot( std_x , std_y , linewidth=2 , ls='--' , color='c')
        #print( hist )
        
        pyplot.axis([-92, 0, 0, numpy.max(hist)*1.05 ])
        
        text_rel_pos = 0.9
        pyplot.text( -85 , numpy.max(hist)*text_rel_pos        ,"mean:    %.3f dB"%rms_mean , fontsize=15,)
        pyplot.text( -85 , numpy.max(hist)*(text_rel_pos-0.05) ,"std dev:  %.3f dB"%rms_std , fontsize=15,)
        pyplot.text( -85 , numpy.max(hist)*(text_rel_pos-0.10) ,"peak:     %.3f dB"%abs_peak , fontsize=15,)
        pyplot.text( -85 , numpy.max(hist)*(text_rel_pos-0.15) ,"max rms:  %.3f dB"%rms_max , fontsize=15,)
        
        pyplot.xlabel('RMS dB')
        pyplot.ylabel('Probability')
        
        if title != None:
            hist_title = title
        else:
            hist_title = "Hystogram of dynamic"
        
        pyplot.title(r'%s'%hist_title)
        
        pyplot.plot( mean_x , mean_y )
        
        pyplot.grid(True)
        pyplot.show()
        
    else:
        ( hist , bin_edges ) = numpy.histogram( rms , bins=bins , normed=True )
             
    return ( hist , bin_edges )
    
    


# dr14_t.meter: compute the DR14 value of the given audiofiles
# Copyright (C) 2011  Simone Riva
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from dr14tmeter.duration import StructDuration
from dr14tmeter.compute_dr14 import compute_dr14
from dr14tmeter.compute_drv import compute_DRV

import numpy

class ComputeAudio :
    def __init__(self):
        self.duration = StructDuration() ;
        self.Dr_lr = numpy.zeros(2)
    
    def compute( self , Y , Fs ):
        pass
    
    def get_duration( self ) :
        return self.duration
    
    def get_dr_lr( self ) :
        return self.Dr_lr
    

class ComputeDR14( ComputeAudio ) :
    def compute( self , Y , Fs ):
       return compute_dr14(  Y , Fs , self.duration , self.Dr_lr )
       

class ComputeDRV( ComputeAudio ) :
    def compute( self , Y , Fs ):
       return compute_DRV(  Y , Fs , self.duration , self.Dr_lr )


# dr14_t.meter: compute the DR14 value of the given audiofiles
# Copyright (C) 2011  Simone Riva
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import numpy
from dr14tmeter.audio_math import *
from dr14tmeter.out_messages import *
from dr14tmeter.my_time_formatter import *

import math
import time

try:
    import matplotlib 
    import matplotlib.pyplot as pyplot
    import matplotlib.mlab as mlab
except:
    ____foo = None
    

def dynamic_vivacity( Y , Fs , Plot=True ):
    
    time_a = time.time()
        
    s = Y.shape
    
    if len( Y.shape ) > 1 :
        ch = s[1]
    else :
        ch = 1
        
    block_size = 1.0
    
    samples_per_block = int( Fs * block_size )
    
    seg_cnt = int( math.floor( s[0] / samples_per_block ) + 1 )
    
    seg_dyn = zeros(( seg_cnt , ch ))
    
    
    curr_sample = 0 
    for i in range( seg_cnt - 1 ):
        #r = numpy.arange( curr_sample , curr_sample + samples_per_block )
        rms = u_rms( Y[curr_sample : curr_sample + samples_per_block , : ] )
        mx = numpy.max( Y[ curr_sample : curr_sample + samples_per_block  ,:] , 0 )
        seg_dyn[i,:] = decibel_u( mx , rms )
        iz = ( rms < audio_min() )
        seg_dyn[i,iz] = 0.0
        curr_sample = curr_sample + samples_per_block
        
    i = seg_cnt - 1 ;
    r = numpy.arange( curr_sample , s[0] )
    
    if r.shape[0] > 0:
        rms = u_rms( Y[r,:] )
        mx = numpy.max( Y[r,:] )
        seg_dyn[i,:] = decibel_u( mx , rms )
        iz = ( rms < audio_min() )
        seg_dyn[i,iz] = 0.0
        
    
    t = numpy.arange( 0.0 , math.floor((1.0/Fs) * s[0]) + 1 , step=block_size  )
        
    max_db = numpy.max( seg_dyn )
    
    non_zeros_ = seg_dyn >= audio_min()
    
    non_zeros = numpy.ones( ( seg_dyn.shape[0] ) ) == 1
    
    for i in range( ch ):
        non_zeros = numpy.logical_and( non_zeros , non_zeros_[:,i] ) 
        
    mean = numpy.mean( seg_dyn[non_zeros,:] , 0 )
    std = numpy.std( seg_dyn[non_zeros,:] , 0 )
    
    tot_t = s[0]*1.0/Fs 
    
    time_b = time.time()
    dr14_log_info( "dynamic_vivacity: Clock: %2.8f" % (time_b - time_a ) )
    
    if Plot :
        for j in range( ch ):
            
            ax = pyplot.subplot( 210+j+1 )
            pyplot.plot( t.T , seg_dyn[:,j] , linewidth=2 , color = "b" )
            pyplot.grid(True)
            
            time_x = numpy.array( [ 0 , tot_t ] )
            mean_y = numpy.array( [ mean[j] , mean[j] ] )
            
            pyplot.plot( time_x , mean_y , linewidth=2 , color="g" )
            
            ax.xaxis.set_major_formatter( MyTimeFormatter() )
            pyplot.xticks(rotation=25)
            
            std_a_y = numpy.array( [ mean[j] , mean[j] ] + std[j] )
            std_b_y = numpy.array( [ mean[j] , mean[j] ] - std[j] )
            
            pyplot.plot( time_x , std_a_y , linewidth=2 , ls='--' , color='c' )
            pyplot.plot( time_x , std_b_y , linewidth=2 , ls='--' , color='c' )
            
            pyplot.axis( [ 0 , tot_t , 0 , max_db * 1.15 ] )
            
            text_rel_pos = 0.2
            pyplot.text( tot_t * 0.1 , max_db* text_rel_pos       , "mean:     %.3f dB"%mean[j] , fontsize=12)
            pyplot.text( tot_t * 0.1 , max_db*(text_rel_pos-0.07) , "std dev:  %.3f dB"%std[j] , fontsize=12)
                        
            pyplot.title( "Channel %d" % (j+1) )
            pyplot.xlabel('Time [min:sec]')
            pyplot.ylabel('Dynamic. [dB]')
            
        
        pyplot.show()


    
    return ( mean , std , seg_dyn )


# dr14_t.meter: compute the DR14 value of the given audiofiles
# Copyright (C) 2011  Simone Riva
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import numpy
from dr14tmeter.audio_math import *
from dr14tmeter.out_messages import *

import math
import time

try:
    import matplotlib 
    import matplotlib.pyplot as pyplot
    import matplotlib.mlab as mlab
except:
    ____foo = None


try:
    class MyTimeFormatter( matplotlib.ticker.Formatter ):
        
        def __init__(self,utime=1.0 , milli_sec=False ):
            self.utime = utime
            self.milli_sec = milli_sec
        
        def __call__( self , x , pos=None ):
            minu = int( self.utime*x / 60 ) ;
            sec = int( self.utime*x - minu*60 )
            msec =  int ( 1000 * ( self.utime*x - int( self.utime*x ) ) )
            
            if self.milli_sec :
                return "%02d:%02d.%03d" % ( minu , sec , msec )
            else :
                return "%02d:%02d" % ( minu , sec )
except:
    class MyTimeFormatter:
        def __init__(self):
            raise
    



# dr14_t.meter: compute the DR14 value of the given audiofiles
# Copyright (C) 2012  Simone Riva
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import numpy as np
from dr14tmeter.audio_math import *
from dr14tmeter.out_messages import *
from dr14tmeter.my_time_formatter import *

import math
import time
import datetime

try:
    import matplotlib
    import matplotlib.pyplot as pyplot
    import matplotlib.mlab as mlab
    from matplotlib import dates
    from matplotlib.widgets import  *
    from pylab import *
except:
    ____foo = None

  
def on_select(vmin, vmax):
    dr14_log_debug( "on_select: %f , %f " % (vmin , vmax) )
    if vmin == vmax :
        return 
    plot_track_classic( plot_str=on_select.plot_str , start_time=vmin , end_time=vmax )
    on_select.plot_str.plot()
 

def wheel_moved( event ) :
    dr14_log_debug( "mouse_pressed: step: %d " % event.step )
    
    ( start_time , end_time ) = wheel_moved.plot_str.move( event.step )
    plot_track_classic( plot_str=wheel_moved.plot_str , start_time=start_time , end_time=end_time )
    wheel_moved.plot_str.plot()

   
def mouse_pressed( event ) :
    
    dr14_log_debug( "mouse_pressed: " )
    
    if event.button == 3 :
        dr14_log_debug( "mouse_pressed: button 3" )
        
        ( start_time , end_time ) = mouse_pressed.plot_str.zoom_out()              
        plot_track_classic( plot_str=mouse_pressed.plot_str , start_time=start_time , end_time=end_time )
        mouse_pressed.plot_str.plot()
        
        dr14_log_debug( "mouse_pressed: start - end:  %f %f " % ( start_time , end_time ) )
        

class PltTrackStruct:
    def __init__( self , plot_mode='fill' , Y = [] , Fs=44100 , sz=0 , ch=0 ):
        self.ch = ch
        self.tot_time = 0
        
        self.t = None
        self.tb = None
        self.start_time = 0 
        self.end_time = 0
        
        self.start_block = 0  
        self.end_block = 0 
        
        self.mp = None
        self.mn = None
            
        self.Y = Y
        self.Fs = Fs
        self.plot_mode = plot_mode # "fill" or "curve"
        self.ax = None
        self.rebuild = False
        self.first_sample = 0
        self.sz_section = 0
        self.lines = []
        self.fig = None


    def start( self ) :
        self.plot()
        pyplot.show()        

    def zoom_out( self ) :
        start_time = self.start_time
        end_time   = self.end_time
        
        m = start_time + ( end_time - start_time ) / 2.0
        new_tot_time = 2.0 * ( end_time - start_time )
        
        start_time = m - new_tot_time/2.0
        end_time = m + new_tot_time/2.0
        
        return ( start_time , end_time )
    
    
    def move( self , sign ):
        
        factor = 0.05
        
        delta = self.end_time - self.start_time
        
        start_time = self.start_time + sign * delta * factor
        end_time   = self.end_time + sign * delta * factor
        
        if start_time < 0 or end_time > self.tot_time:
            start_time = self.start_time 
            end_time   = self.end_time
        
        return ( start_time , end_time )
    

    def plot( self ):
        
        new_flag = False
        
        if self.fig == None :
            self.fig = pyplot.figure()
            self.ax = [i for i in range(self.ch)]
            self.lines = [i for i in range(self.ch)]
            self.span = []
            new_flag = True
            
            
        for j in range( self.ch ):
            
            if new_flag :
                self.ax[j] = self.fig.add_subplot( 210+j+1 )
            
            if type( self.lines[j] ) is matplotlib.collections.PolyCollection :
                self.lines[j].remove()
            else:
                while ( not new_flag ) and len( self.lines[j] ) > 0:
                    l = self.lines[j].pop(0)
                    l.remove()
            
            if self.plot_mode == 'fill' :
                self.lines[j] = self.ax[j].fill_between( self.tb[ self.start_block:self.end_block ] ,
                                                         self.mp[ self.start_block:self.end_block , j ] ,
                                                         self.mn[ self.start_block:self.end_block , j ]  ) 
            else:
                self.lines[j] = self.ax[j].plot( self.t , self.Y[ self.first_sample : self.first_sample+self.sz_section , j ] , 'b' ) 
            
            self.ax[j].axis( [ self.start_time , self.end_time , -1.0 , 1.0 ] )
            
            delta_time = self.end_time - self.start_time
            
            if delta_time < 1.5 :
                milli_sec=True 
            else :
                milli_sec=False
            
            self.ax[j].xaxis.set_major_formatter( MyTimeFormatter( milli_sec=milli_sec ) )
            
            self.ax[j].grid(True)
            
            pyplot.title( "Channel %d" % (j+1) )
            pyplot.xlabel('Time [min:sec]')
            pyplot.ylabel('Amplitude')
        
            if new_flag :
                onsel = on_select
                onsel.plot_str = self
                self.span.append( SpanSelector( self.ax[j], onsel, 'horizontal' ) )
                
                m_p = mouse_pressed
                m_p.plot_str = self
                connect('button_press_event', m_p)
                
                w_m = wheel_moved
                w_m.plot_str = self
                connect('scroll_event', w_m)
            
        self.fig.canvas.draw()
        
 
    

def plot_track_classic( Y=None , Fs=None , plot_str=None , utime=0.02 , time_lim=5 , start_time=0.0 , end_time = -1.0 ):
        
    if Y == None and Fs == None and plot_str == None :
        raise
    
    if plot_str == None and Y != None and Fs != None :        
        s = Y.shape
        
        if len( Y.shape ) > 1 :
            ch = s[1]
        else :
            ch = 1           
        
        plot_str = PltTrackStruct( ch=ch )
        plot_str.Y = Y
        plot_str.Fs = Fs
        plot_str.tot_time = s[0] * 1.0/Fs
        
    elif plot_str != None :
        Fs = plot_str.Fs
        Y = plot_str.Y
        
        s = Y.shape
        if len( Y.shape ) > 1 :
            ch = s[1]
        else :
            ch = 1
    else:
        raise
     
    
    sz_section = s[0]
    first_sample = 0
    
    if end_time > s[0] * 1.0/Fs :
        end_time = s[0] * 1.0/Fs
        
    if start_time < 0 :
        start_time = 0.0
    
    if start_time >= 0.0 and end_time > start_time :
        sz_section = int( ( end_time - start_time ) * Fs )
        first_sample = int( start_time * Fs )
        plot_str.start_time = start_time
        plot_str.end_time = end_time
    else :
        plot_str.start_time = 0.0
        plot_str.end_time = sz_section * 1.0/Fs
            
    
    block_len = utime
    
    samples_block = block_len * Fs
    sz = int( sz_section / samples_block )
    
    curr_sample = first_sample
    
    if plot_str.mp != None and plot_str.mn != None and sz_section >= time_lim*Fs :
        plot_str.plot_mode = "fill"
        plot_str.start_block = int( plot_str.start_time / block_len )
        plot_str.end_block   = int( plot_str.end_time / block_len )
        
        return plot_str
        
    if sz_section > time_lim*Fs:
        plot_str.plot_mode = "fill"
        plot_str.mp = zeros( ( sz , ch ) )
        plot_str.mn = zeros( ( sz , ch ) )
        plot_str.tb = start_time + np.arange( 0 , sz ) * block_len
        
        for i in range( sz ):
            plot_str.mp[i,:] = np.max( Y[curr_sample:curr_sample+samples_block,:] , 0 )
            plot_str.mn[i,:] = np.min( Y[curr_sample:curr_sample+samples_block,:] , 0 )
            curr_sample = curr_sample + samples_block
        
        plot_str.start_block = 0
        plot_str.end_block = sz
        
    else:
        plot_str.plot_mode = "curve"
        plot_str.t = start_time + np.arange( 0 , sz_section ) * 1.0/Fs
        plot_str.first_sample = first_sample
        plot_str.sz_section = sz_section
        
    return plot_str
    
    
    
    
    


# dr14_t.meter: compute the DR14 value of the given audiofiles
# Copyright (C) 2011  Simone Riva
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.



from dr14tmeter.audio_math import *
from dr14tmeter.out_messages import *
import math
import numpy
import time


def compute_dr14( Y , Fs , duration = None , Dr_lr = None ) :

    s = Y.shape
    
    if len( Y.shape ) > 1 :
        ch = s[1]
    else :
        ch = 1

    if Fs == 44100:
        delta_fs = 60
    else:
        delta_fs = 0

    dr14_log_debug( "compute_dr14: Y: Fs: %d ; ch: %d ; shape: %d " %(Fs , ch , s[0]) )
    time_a = time.time()
    
    block_time = 3
    cut_best_bins = 0.2
    block_samples = block_time * ( Fs + delta_fs )

    seg_cnt = int( math.floor( s[0] / block_samples ) + 1 )

    if seg_cnt < 1:
        dr14_log_debug( "compute_dr14: EXIT - too short" )
        return ( 0 , -100 , -100 )

    curr_sam = 0
    rms = zeros((seg_cnt,ch))
    peaks = zeros((seg_cnt,ch))
    
    
    for i in range(seg_cnt - 1):
        rms[i,:] = numpy.sqrt( 2.0 * numpy.sum( Y[curr_sam:curr_sam+block_samples,:]**2.0 , 0 ) / float(block_samples) ) 
        peaks[i,:] = numpy.max( numpy.abs( Y[curr_sam:curr_sam+block_samples,:] ) , 0 )
        curr_sam = curr_sam + block_samples

    i = seg_cnt - 1 ;

    if curr_sam < s[0]:
        rms[i,:] = dr_rms( Y[curr_sam:s[0]-1,:] )
        peaks[i,:] = numpy.max( numpy.abs( Y[curr_sam:s[0]-1,:] ) , 0 )

    peaks = numpy.sort( peaks , 0 )
    rms = numpy.sort( rms , 0 )
    
    n_blk = int( floor( seg_cnt * cut_best_bins ) )
    if n_blk == 0:
        n_blk = 1

    r = arange(seg_cnt-n_blk,seg_cnt)
    
    rms_sum = numpy.sum( rms[r,:]**2 , 0 )

    ch_dr14 = -20.0 * numpy.log10( numpy.sqrt( rms_sum / n_blk ) * 1.0/peaks[seg_cnt-2,:] )

    err_i = numpy.logical_or( rms_sum < audio_min() , numpy.abs( ch_dr14 ) > max_dynamic(24) )
    ch_dr14[err_i] = 0.0 ;

    dr14 = round( numpy.mean( ch_dr14 ) )

    dB_peak = decibel_u( numpy.max( peaks ) , 1.0 )

    y_rms = numpy.sum( numpy.mean( rms , 0 ) ) / 2.0 

    dB_rms = decibel_u( y_rms , 1 )
    
    if duration != None :
        duration.set_samples( s[0] , Fs )
        
    if Dr_lr != None :
        Dr_lr = ch_dr14

    time_b = time.time()
    dr14_log_info( "compute_dr14: Clock: %2.8f" % (time_b - time_a ) )

    return ( dr14 , dB_peak , dB_rms )
    



# dr14_t.meter: compute the DR14 value of the given audiofiles
# Copyright (C) 2011  Simone Riva
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import tempfile
import subprocess
import re
import wave
import numpy
from dr14tmeter.audio_file_reader import *


class AudioDecoder:

    def __init__(self):
        self.formats = [ '.flac' , '.mp3' , '.ogg' , '.mp4' , '.m4a' , '.wav' , '.ape' , '.ac3', '.wma' ]
    
    def read_track_new( self , file_name , target ):

        ( f , ext ) = os.path.splitext( file_name )
        ext = ext.lower()

        if ext not in self.formats :
            return False

        af = AudioFileReader()
    
        if ext == '.mp3':
            af = Mp3FileReader()
        elif ext == '.flac':
            af = FlacFileReader()
        elif ext == '.ogg':
            af = OggFileReader()
        elif ext in ['.mp4' , '.m4a' ]:
            af = Mp4FileReader()
        elif ext == '.wav':
            af = WavFileReader()
        elif ext == '.ape':
            af = ApeFileReader()
        elif ext == '.ac3':
            af = Ac3FileReader()
        elif ext == '.wma':
            af = WmaFileReader()
        else:
            return False

        #af = PipeFileReader() 

        ret_f = af.read_audio_file_new( file_name , target )

        return ret_f
 




# dr14_t.meter: compute the DR14 value of the given audiofiles
# Copyright (C) 2011  Simone Riva
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
    

import os
import argparse
import time
import multiprocessing
from dr14tmeter.dynamic_range_meter import DynamicRangeMeter
from dr14tmeter.table import *
from dr14tmeter.audio_analysis import *
from dr14tmeter.dr14_global import *
import subprocess
import sys
import re
import tempfile
from dr14tmeter.out_messages import print_msg
import fileinput


def scan_files_list( input_file , options , out_dir ):
    
    success = False
    r = 0 ;
    
    if out_dir == None:
        out_dir = tempfile.gettempdir()
    else:
        out_dir = os.path.abspath( out_dir )
    
    a = time.time()
    
    if input_file == None:
        input_file = '-'
    
    files_list = []
    
    for line in fileinput.input( input_file ):
        files_list.append( os.path.abspath( line.rstrip() ) )
        
    dr = DynamicRangeMeter()
    
    r = dr.scan_mp( files_list=files_list , thread_cnt=get_thread_cnt() )
    
    if r == 0:
        success = False
    else:
        write_results( dr , options , out_dir , "" )
        success = True
    
    clock = time.time() - a
    return (success,clock,r)
    

def scan_dir_list( subdirlist , options , out_dir ):
    a = time.time()

    success = False

    for cur_dir in subdirlist :
        dr = DynamicRangeMeter()
        print_msg ( "\n------------------------------------------------------------ " )		        
        print_msg ( "> Scan Dir: %s \n" % cur_dir )
        
        cpu = multiprocessing.cpu_count()
                
        if ( options.disable_multithread == True ) :
            r = dr.scan_dir( cur_dir )
        else:
            cpu = cpu / 2
            if cpu <= 2:
                cpu = 2
            else:
                cpu = int( round( cpu ) )
                
            r = dr.scan_mp( cur_dir , cpu )
            
        if r == 0:
            continue
        else:
            success = True
            
        
        write_results( dr , options , out_dir , cur_dir )        
         
    
    clock = time.time() - a
    
    return (success,clock,r)


def get_thread_cnt():
    cpu = multiprocessing.cpu_count()
    cpu = cpu / 2
    if cpu <= 2:
        cpu = 2
    else:
        cpu = int( round( cpu ) )
    return cpu


def list_rec_dirs( basedir , subdirlist=None ):
    
    if subdirlist == None :
        subdirlist = []
        subdirlist.append( basedir )
        

    for item in os.listdir( basedir ):
        item = os.path.join( basedir , item )
        if os.path.isdir( item ):
            item = os.path.abspath( item )
            #print_msg( item )
            subdirlist.append( item )
            list_rec_dirs( item , subdirlist )
            
    return subdirlist
        

def write_results( dr , options , out_dir , cur_dir ) :
    out_list = "" ;
   
    table_format = not( options.basic_table )    
    
    if out_dir == None :
        full_out_dir = os.path.join( cur_dir )
    else :
        full_out_dir = out_dir
    
    print_msg( "DR = " + str( dr.dr14 ) )
    
    if not ( os.access( full_out_dir , os.W_OK ) ) :
        full_out_dir = tempfile.gettempdir() ; 
        print_msg( "--------------------------------------------------------------- " )
        print_msg( "- ATTENTION !" )
        print_msg( "- You don't have the write permission for the directory: %s " % full_out_dir )
        print_msg( "- The result files will be written in the tmp dir: %s " % full_out_dir )
        print_msg( "--------------------------------------------------------------- " )
           
    
    if options.print_std_out :
        dr.fwrite_dr( "" , TextTable() , table_format , True )
        
    if options.turn_off_out :
        return 
    
    all_tables = False
    if 'a' in options.out_tables:
        all_tables = True 
    
    tables_list = { 'b' : ["dr14_bbcode.txt",BBcodeTable()] , 't' : ["dr14.txt",TextTable()]  ,
        'h' : ["dr14.html",HtmlTable()] , 'w' : ["dr14_mediawiki.txt",MediaWikiTable()] }
    
    out_list = ""
    
    
    for code in tables_list.keys():
        if code in options.out_tables or all_tables :
            dr.fwrite_dr( os.path.join( full_out_dir , tables_list[code][0] ) , tables_list[code][1] , table_format , append=options.append , dr_database=options.dr_database )
            out_list = out_list + " %s " % tables_list[code][0]
            
    
    print_msg("")
    print_msg("- The full result has been written in the files: %s" % out_list )
    print_msg("- located in the directory: ")
    print_msg( full_out_dir )
    print_msg("")


def run_analysis_opt( options , path_name ):
    
    flag = False
    
    if options.compress:
        
        if test_compress_modules() == False :
            sys.exit(1)
        
        print_msg("Start compressor:")
        comp = AudioCompressor()
        comp.setCompressionModality( options.compress )
        comp.compute_track( path_name )
        flag = True
        
    
    if options.spectrogram:
        
        if test_matplotlib_modules( "Spectrogram" ) == False:
            sys.exit(1)
        
        print_msg("Start spectrogram:")
        spectr = AudioSpectrogram()
        spectr.compute_track( path_name )
        flag = True

    if options.plot_track:
        
        if test_matplotlib_modules( "Plot track" ) == False:
            sys.exit(1)
        
        print_msg("Start Plot Track:")
        spectr = AudioPlotTrack()
        spectr.compute_track( path_name )
        flag = True

    if options.plot_track_dst:
        
        if test_matplotlib_modules( "Plot track dst" ) == False:
            sys.exit(1)
        
        print_msg("Start Plot Track:")
        spectr = AudioPlotTrackDistribution()
        spectr.compute_track( path_name )
        flag = True

    if options.histogram:
        
        if test_hist_modules() == False:
            sys.exit(1)
        
        print_msg("Start histogram:")
        hist = AudioDrHistogram()
        hist.compute_track( path_name )
        flag = True

    if options.lev_histogram:
        
        if test_hist_modules() == False:
            sys.exit(1)
        
        print_msg("Start level histogram:")
        hist = AudioLevelHistogram()
        hist.compute_track( path_name )
        flag = True
    
        
    if options.dynamic_vivacity :
        if test_hist_modules() == False:
            sys.exit(1)
        
        print_msg("Start Dynamic vivacity:")
        viva = AudioDynVivacity()
        viva.compute_track( path_name )
        flag = True
        
    return flag



# dr14_t.meter: compute the DR14 value of the given audiofiles
# Copyright (C) 2011  Simone Riva
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from numpy  import *
import math
import numpy


def dr_rms( y ) :
    n = y.shape
    return numpy.sqrt( 2.0 * numpy.sum( y**2.0 , 0 ) / float(n[0]) )

def u_rms( y ) :
    n = y.shape
    return numpy.sqrt( numpy.sum( y**2.0 , 0 ) / float(n[0]) )


def decibel_u( y , ref ) :
    return 20.0 * numpy.log10( y / ref )


def decibel_p( y , ref ) :
    return 10.0 * numpy.log10( y / ref )
    
def audio_min() :
    return 1.0/(2.0**24)

def audio_min16():
    return 1.0/(2.0**16)

def max_dynamic( bit ):
    return 20.0*numpy.log10( 2.0**bit )

def normalize( y , ml=1.0 ):
    m = numpy.max( numpy.abs( y ) )
    return ml * ( y * (1.0/m) )


# dr14_t.meter: compute the DR14 value of the given audiofiles
# Copyright (C) 2011 - 2012  Simone Riva
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

class StructDuration:
    def __init__( self ):
        self.tm_min = 0
        self.tm_sec = 0
    
    def set_samples( self , samples , Fs ) :
        mint = samples * (1.0 / Fs) / 60.0 
        self.tm_min = int( mint )
        self.tm_sec = int( ( mint - self.tm_min ) * 60.0 )
    
    def to_str( self ):
        return str( self.tm_min ) + ":%02d" % int(self.tm_sec)
    
    def to_float( self ):
        return float( self.tm_min ) + float(int(self.tm_sec))/100.0
    
    def float_to_str( self , f ):
        return "%d:%02d" % ( int(f) , int( 100*( f - int(f) ) ) )
        



# dr14_t.meter: compute the DR14 value of the given audiofiles
# Copyright (C) 2011  Simone Riva
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import sys
import re
import threading
import subprocess

from dr14tmeter.out_messages import print_msg


###########################
# Current version
v_major    = 1
v_minor    = 0
v_revision = 16
###########################

# latest version
l_major = 0
l_minor = 0
l_revision = 0

lock_ver = threading.Lock()

ffmpeg_cmd = None


def dr14_version():
    global v_major
    global v_minor
    global v_revision
    return "%d.%d.%d" % ( v_major , v_minor , v_revision )

def min_dr() :
    return -10000

def get_exe_name():
    return "dr14_tmeter"

def get_ffmpeg_cmd():
    
    global ffmpeg_cmd
    
    if ffmpeg_cmd == None :
        ffmpeg_f = subprocess.call("type " + "ffmpeg" , shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE ) == 0
        avconv_f = subprocess.call("type " + "avconv" , shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE ) == 0
        
        if ffmpeg_f :
            ffmpeg_cmd = "ffmpeg"
        elif avconv_f :
            ffmpeg_cmd = "avconv"
            
    return ffmpeg_cmd
        

class TestVer(threading.Thread):
    def run(self):
        _dr14_get_latest_version()


def _dr14_get_latest_version():
    
    global l_major
    global l_minor
    global l_revision
    global lock_ver
    
    ver_url = "http://simon-r.github.com/dr14_t.meter/ver.html"
    #print_msg ( ver_url )
    
    try:
        if sys.version_info[0] > 2 :
            import urllib.request
            opener = urllib.request.FancyURLopener({})
            f = opener.open( ver_url )
            vr = f.read()
            vr = vr.decode()
        else :
            import urllib
            opener = urllib.FancyURLopener({})
            f = opener.open( ver_url )
            vr = f.read()
    except:
        return 
    
    re_flags = ( re.MULTILINE | re.IGNORECASE | re.UNICODE )
    m = re.search( r"<div\s+id=\"version\">\s*(\d+)\.(\d+)\.(\d+)\s*</div>" , vr , re_flags )
    
    if m == None:
        return 
    
    
    lock_ver.acquire()
    if len( m.groups() ) == 3 :
        l_major = int( m.groups()[0] )
        l_minor = int( m.groups()[1] )
        l_revision = int( m.groups()[2] )
    else :
        l_major = 0
        l_minor = 0
        l_revision = 0
        
    lock_ver.release()
    

    
def test_new_version():
    global l_major
    global l_minor
    global l_revision
    
    global v_major
    global v_minor
    global v_revision
    
    lock_ver.acquire()
    
    curr_ver = ( v_major , v_minor , v_revision )
    online_ver = ( l_major , l_minor , l_revision )
    
    for i in range( 0 , 3 ) :
        if online_ver[i] > curr_ver[i] :
            lock_ver.release()
            return True
        elif online_ver[i] < curr_ver[i] :
            lock_ver.release()
            return False
    
    lock_ver.release()
    return False
    

def get_new_version():
    
    global lock_ver
    
    lock_ver.acquire()
    res = "%d.%d.%d" % ( l_major , l_minor , l_revision )
    lock_ver.release()
    
    return res
    

def get_home_url():
    return "http://simon-r.github.com/dr14_t.meter"
        
      
      
def test_matplotlib_modules( fun_name ):
    try:
        import matplotlib
    except ImportError:
        print_msg( "The %s function require the installation of MatPlotLib: http://matplotlib.sourceforge.net" % fun_name )
        return False
    
    return True


def test_hist_modules():
    try:
        import matplotlib
    except ImportError:
        print_msg( "The histogram function require the installation of MatPlotLib: http://matplotlib.sourceforge.net" )
        return False
    
    return True

def test_compress_modules():
    try:
        import scipy
    except:
        print_msg( "The compression function require the installation of SciPy: http://www.scipy.org/" )
        return False
    
    return True
        



#!/usr/bin/python

# dr14_t.meter: compute the DR14 value of the given audiofiles
# Copyright (C) 2011  Simone Riva
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
    

import os
import time
import multiprocessing
from dr14tmeter.parse_args import parse_args
from dr14tmeter.dynamic_range_meter import DynamicRangeMeter
from dr14tmeter.table import *
from dr14tmeter.audio_analysis import *
from dr14tmeter.dr14_global import dr14_version, TestVer, test_new_version, get_home_url, get_new_version, get_exe_name, test_hist_modules, test_compress_modules
import subprocess
import inspect
import sys
import re
from dr14tmeter.dr14_utils import *
from dr14tmeter.out_messages import *
import logging
import numpy

    
def main():
        
    
    
    options = parse_args()

    if options.version :
        print_msg( dr14_version() )
        return

    init_log( logging.DEBUG )
    logging.disable( logging.INFO )
    
    numpy.seterr(all='ignore')
    
    #print( options )

    if options.path_name != None:
        path_name = os.path.abspath( options.path_name )
    else:
        path_name = os.path.abspath( '.' )
        
    if not( os.path.exists( path_name ) ) :
        print_msg( "Error: The input directory \"%s\" don't exixst! " % path_name )
        return 

    if options.out_dir and not( os.path.exists( options.out_dir ) ) :
        print_msg( "Error (-o): The target directory \"%s\" don't exixst! " % options.out_dir )
        return 

    if options.quiet :
        set_quiet_msg()

    l_ver = TestVer()
    l_ver.start()
    
    print_msg( path_name )
    print_msg( "" )

    if options.recursive :
        subdirlist = list_rec_dirs( path_name )
    else :
        subdirlist = [] 
        subdirlist.append( path_name )
    
          
    #print ( subdirlist )

    if run_analysis_opt( options , path_name ) :
        return 1

     
    if options.scan_file:
                
        dr = DynamicRangeMeter()
        r = dr.scan_file( path_name )
        
        if r == 1:
            print_out( "" )
            print_out( dr.res_list[0]['file_name'] + " :" )
            print_out( "DR      = %d" % dr.res_list[0]['dr14'] )
            print_out( "Peak dB = %.2f" % dr.res_list[0]['dB_peak'] )
            print_out( "Rms dB  = %.2f" % dr.res_list[0]['dB_rms'] )
            return 1 
        else:
            print_msg( "Error: invalid audio file" )
            return 0


    if options.out_dir == "" :
        out_dir = None
    else :
        out_dir = options.out_dir

    if options.append and out_dir == None:
        out_dir = path_name
    
    if options.files_list:
        (success,clock,r) = scan_files_list(options.path_name,options,out_dir)
    else:    
        (success,clock,r) = scan_dir_list(subdirlist,options,out_dir)
            
    if success :
        print_msg( "Success! " )
        print_msg( "Elapsed time: %2.2f sec" % clock )
    else:
        print_msg("No audio files found\n")
        print_msg(" Usage: %s [options] path_name \n\nfor more details type \n%s --help\n" % ( get_exe_name() , get_exe_name() ) )

    if sys.platform.startswith('linux'):
        subprocess.call( "stty sane" , shell=True ) 

    if test_new_version() :
        print_msg( "\n----------------------------------------------------------------------" )
        print_msg( " A new version of dr14_t.meter [ %s ] is available for download \n please visit: %s" % ( get_new_version() , get_home_url() ) )
        print_msg( "----------------------------------------------------------------------\n" )
    
    return r


if __name__ == '__main__':
    main()




# dr14_t.meter: compute the DR14 value of the given audiofiles
# Copyright (C) 2011  Simone Riva
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import numpy as np
from dr14tmeter.audio_math import *
from dr14tmeter.out_messages import *
from dr14tmeter.my_time_formatter import *

import math
import time
import datetime

try:
    import matplotlib.pyplot as pyplot
    import matplotlib.mlab as mlab
    from matplotlib import dates
except:
    ____foo = None

def f_utime( t ) :
    v = 0.25
    a = 1/400
    xl = 200
    if t > xl :
        return a*t + (v-a*xl)
    else:
        return v
    

def plot_track( Y , Fs , Plot=True , time_range=None , utime = 0 ):
    
    time_a = time.time()
        
    s = Y.shape
    
    if len( Y.shape ) > 1 :
        ch = s[1]
    else :
        ch = 1
    
    ttime = (s[0] * 1/Fs)
    if utime == 0 :
        d = ttime / f_utime(ttime) ;
        utime = ttime / d
        #print( utime )
    
    Fs = int(Fs * utime)
    
    sec = floor( s[0] / Fs ) + 1   
    sz = Fs * ( sec + 1 )
    tm = floor( np.arange(sz) / Fs )
     
    Yc = np.zeros( sz , dtype=np.float32 )
    
    for i in range(ch):
        
        ax = pyplot.subplot( 210+i+1 )
               
        Yc[:] = 0.0
        Yc[0:s[0]] = Y[:,1]
        
        (H, xedges, yedges) = np.histogram2d( tm , Yc , bins=( sec , 500 ))
        
        mh = np.max( H , 1 )
        H = (H.T * (1/mh))
                
        ax.xaxis.set_major_formatter( MyTimeFormatter( utime ) )
        pyplot.xticks(rotation=25)
        
        extent = [xedges[0], xedges[-1], yedges[-1], yedges[0]]
        pyplot.imshow(H, extent=extent, interpolation='nearest',aspect='auto' , cmap="hot" )

        
    pyplot.show()        
    
    
    time_b = time.time()
    dr14_log_info( "plot_track: Clock: %2.8f" % (time_b - time_a ) )
    
    


# dr14_t.meter: compute the DR14 value of the given audiofiles
# Copyright (C) 2011 - 2012  Simone Riva
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import dr14tmeter.dr14_global as dr14
import dr14tmeter.table as table

class WriteDr :
    
    def __init__(self):
        self.__dr_database_compatible = True
    
    def set_dr_database( self , f ):
        self.__dr_database_compatible = f
        
    def get_dr_database(self):
        return self.__dr_database_compatible
    
    def write_dr( self , drm , tm ):
        txt = ""
        
        ( head , album_dir ) = os.path.split( drm.dir_name )
        
        txt = tm.new_table(txt)
        
        txt = tm.new_head( txt )
        
        txt = tm.append_separator_line( txt )
        txt = tm.add_title( txt , " Analyzed folder:  " + album_dir )
        
        txt = tm.end_head( txt )
        
        txt = tm.new_tbody( txt )
        
        txt = tm.append_separator_line( txt )
        txt = tm.append_row( txt , [ " DR", "Peak", "RMS", "Duration" , "File name" ] , 'h' )
        txt = tm.append_separator_line( txt )
        
        for i in range( len( drm.res_list ) ) :
            
            if drm.res_list[i]['dr14'] > dr14.min_dr() :
                row = []
                row.append( " DR%d" % drm.res_list[i]['dr14'] )
                row.append( " %.2f" % drm.res_list[i]['dB_peak'] + ' dB' )
                row.append( " %.2f" % drm.res_list[i]['dB_rms'] + ' dB' )
                row.append( " %s" % drm.res_list[i]['duration'] )
                row.append( " %s" % drm.res_list[i]['file_name'] )
            
                txt = tm.append_row( txt , row )

        txt = tm.end_tbody( txt )
        
        txt = tm.new_foot( txt )
        txt = tm.append_separator_line( txt )
               
        txt = tm.add_title( txt , "Number of files:    " + str(len( drm.res_list )) )
        txt = tm.add_title( txt , "Official DR value:  DR%d" % int(drm.dr14) )
        
        txt = tm.append_empty_line( txt )
        txt = tm.add_title( txt , "Dr14 T.meter %s " % dr14.dr14_version() )
        
        txt = tm.append_closing_line( txt )
        txt = tm.end_foot( txt )
        
        txt = tm.end_table(txt)
        
        return txt 
    

class WriteDrExtended( WriteDr ) :

    def __init__(self):
        WriteDr.__init__(self)

    def write_dr( self , drm , tm ):
        txt = ""
         
        ( head , album_dir ) = os.path.split( drm.dir_name )
        
        txt = tm.new_table( txt )
        
        txt = tm.new_head( txt )
        txt = tm.append_separator_line( txt )
        
        album_t = drm.meta_data.get_album_title()
        artist = drm.meta_data.get_album_artist()
        
        if not isinstance( tm , table.TextTable ) :
            self.set_dr_database( False )
        
        if self.get_dr_database() :
        
            title = "" 
            
            if album_t == None :
                title = " Analyzed folder:  " + album_dir 
            else:
                title = " Analyzed: " + album_t 
                if artist != None :
                    title = title + " /  Artist: " + artist
            txt = tm.add_title( txt , title )
        
        else:
        
            if album_t == None :
                txt = tm.add_title( txt , " Analyzed folder:  " + album_dir )
            else:
                txt = tm.add_title( txt , " Album: " + album_t )
                if artist != None :
                    txt = tm.add_title( txt , " Artist: " + artist )
            
        
        txt = tm.end_head( txt )
        
        txt = tm.new_tbody( txt )
        
        txt = tm.append_separator_line( txt )
        txt = tm.append_row( txt , [ "DR", "Peak", "RMS", "Duration" , "Title [codec]" ] , 'h' )
        txt = tm.append_separator_line( txt )
        
        list_bit = []
        
        sum_kbs = 0
        cnt = 0
        
        sampl_rate = []
        
        d_nr = 0 ;        
        
        
        for i in range( len( drm.res_list ) ) :
            
            if drm.res_list[i]['dr14'] > dr14.min_dr() :
                row = []
                row.append( " DR%d" % drm.res_list[i]['dr14'] )
                row.append( " %.2f" % drm.res_list[i]['dB_peak'] + ' dB' )
                row.append( " %.2f" % drm.res_list[i]['dB_rms'] + ' dB' )
                row.append( drm.res_list[i]['duration'] )
                
                #print( "> " + drm.res_list[i]['file_name'] )
                
                curr_file_name = drm.res_list[i]['file_name']
                
                tr_title = drm.meta_data.get_value( curr_file_name , 'title' )
                #print( "> " + tr_title )
                if tr_title == None :
                    row.append( drm.res_list[i]['file_name'] )
                else:
                    nr = drm.meta_data.get_value( curr_file_name , 'nr' )
                    codec = drm.meta_data.get_value( curr_file_name , 'codec' )
                    
                    if nr == None :
                        nr = i + 1
                    
                    row.append( "%02d - %s \t [%s]" % ( nr , tr_title , codec ) )
                    
                bitrate = drm.meta_data.get_value( curr_file_name , 'bitrate' )
                bit = drm.meta_data.get_value( curr_file_name , 'bit' )
                s_rate = drm.meta_data.get_value( curr_file_name , 's_rate' )
                
                kbs = drm.meta_data.get_value( curr_file_name , 'bitrate' )
                
                if kbs != None :
                    sum_kbs += int( kbs )
                    cnt = cnt + 1
                    
                if bit not in list_bit :
                    list_bit.append( bit )    
                        
                if s_rate not in sampl_rate :
                    sampl_rate.append( s_rate )
                    
                txt = tm.append_row( txt , row )
        
        txt = tm.end_tbody( txt )
        
        txt = tm.new_foot( txt )
        txt = tm.append_separator_line( txt )
               
        txt = tm.add_title( txt , " Number of files:    " + str(len( drm.res_list )) )
        txt = tm.add_title( txt , " Official DR value:  DR%d" % int(drm.dr14) )
        
        txt = tm.append_empty_line( txt )
        
        txt = tm.add_title( txt , " Sampling rate: \t\t %s Hz" % sampl_rate[0] )
        
        if cnt > 0:
            txt = tm.add_title( txt , " Average bitrate: \t\t %dkbs " % ( sum_kbs / cnt )  )
        
        mf_bit = max( set( list_bit ) , key=list_bit.count )
        txt = tm.add_title( txt , " Bits per sample: \t\t %s bit" % ( mf_bit ) )
        
        txt = tm.append_empty_line( txt )
        txt = tm.add_title( txt , "Dr14 T.meter %s " % dr14.dr14_version() )
        
        txt = tm.append_closing_line( txt )
        txt = tm.end_foot( txt )
        
        txt = tm.end_table(txt)
        
        return txt

 


# dr14_t.meter: compute the DR14 value of the given audiofiles
# Copyright (C) 2011  Simone Riva
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import subprocess
import sys
import os
import re

from dr14tmeter.audio_decoder import AudioDecoder
from dr14tmeter.dr14_global import get_ffmpeg_cmd

# Test example !!!!!
# a = subprocess.check_output( [ "ffprobe" , "-show_format" , "/media/esterno_xfs/data/Musica/Musica/aavv/01-blitzkrieg_bop_160_lame_abr.mp3" ] , stderr=subprocess.STDOUT , shell=False )


class RetirveMetadata:
    
    def __init__( self ):
        self._album = {}
        self._artist = {}
        self._tracks = {}
        
        if get_ffmpeg_cmd() == "ffmpeg" :
            self.__ffprobe_cmd = "ffprobe"
        elif get_ffmpeg_cmd() == "avconv" :
            self.__ffprobe_cmd = "avprobe"        
    
    
    def scan_dir( self , dir_name , files_list=None ):
        
        self._album = {}
        self._tracks = {}
        self._artist = {}
        
        if files_list == None:
            dir_name = os.path.abspath( dir_name )
            files_list = sorted( os.listdir( dir_name ) )
        
        ad = AudioDecoder()
        
        for file_name in files_list :
            
            ( fn , ext ) = os.path.splitext( file_name )
            full_file = os.path.join( dir_name , file_name )
            
            if ext in ad.formats :
                self.scan_file( full_file )
                
        #print( self._tracks )
    
    
    def scan_file( self , file_name ):
        
        #print("")
        #print( file_name )
        
        try:
            data_txt = subprocess.check_output( [ self.__ffprobe_cmd , "-show_format" , file_name ] , stderr=subprocess.STDOUT , shell=False )
        except :
            data_txt = "ffprobe ERROR"
         
        if data_txt != "ffprobe ERROR" :
            try:
                data_txt = data_txt.decode(encoding='UTF-8')
            except:
                data_txt = data_txt.decode(encoding='ISO-8859-1')
        
        track = {} 
        
        re_flags = ( re.MULTILINE | re.IGNORECASE | re.UNICODE )
        
        m = re.search( r"\s*track\s*\:\s*(\d+)$" , data_txt , re_flags )
        if m != None:
            track['nr'] = int( m.group(1) )
        
        m = re.search( r"\s*album\s*\:\s*(.*)$" , data_txt , re_flags )
        if m != None:
            #print( m.group(1) )
            self._album.setdefault( m.group(1) , 0 )
            self._album[m.group(1)] += 1
            
        m = re.search( r"\s*title\s*\:\s*(.*)$" , data_txt , re_flags )
        if m != None:
            track['title'] = m.group(1)
        
        m = re.search( r"\s*artist\s*\:\s*(.*)$" , data_txt , re_flags )
        if m != None:
            self._artist.setdefault( m.group(1) , 0 )
            self._artist[m.group(1)] += 1
        
        m = re.search( r"\s*genre\s*\:\s*(.*)$" , data_txt , re_flags )
        if m != None:
            track['genre'] = m.group(1)
         
        ##########################################
        # string examples:   
        #Audio: flac, 44100 Hz, stereo, s16
        #Stream #0:0(und): Audio: alac (alac / 0x63616C61), 44100 Hz, 2 channels, s16, 634 kb/s
        #Stream #0:0: Audio: flac, 44100 Hz, stereo, s16
        m = re.search( r"\:\s*Audio\s*\:\s*(\w+)[^,]*,\s*(\d*)\s*Hz\s*,\s*([\w\s]*)\s*,\s*s(\d+)" , data_txt , re_flags )
        if m != None:
            track['codec'] = m.group(1)
            track['s_rate'] = m.group(2)
            track['channel'] = m.group(3)
            track['bit'] = m.group(4)
            
            #print ( m.group(1) + " " + m.group(2)+ " " + m.group(3)+ " " + m.group(4) )
        
        m = re.search( r"\,\s*bitrate\s*\:\s*(\d*)\s*kb" , data_txt , re_flags )
        if m != None:
            track['bitrate'] = m.group(1)
            #print ( m.group(1) )
            
        ( foo , f_key ) = os.path.split( file_name )
        self._tracks[f_key] = track 
        


    def album_len( self ):
        return len( self._tracks )


    def get_album_title( self ):

        if len( self._album ) > 1 :
            return "Various"
        elif len( self._album ) == 0 :
            return None
        else :
            for k in self._album.keys():
                res = k
            return res

    
    def get_album_artist( self ):

        if len( self._artist ) > 1 :
            return "Various Artists"
        elif len( self._artist ) == 0 :
            return None
        else :
            for k in self._artist.keys():
                res = k
            return res
    
    def get_value( self , file_name , field ):
        
        f = self._tracks.get( file_name , None )
        
        if f == None :
            return None
        
        return f.get( field , None )
        


        




# dr14_t.meter: compute the DR14 value of the given audiofiles
# Copyright (C) 2011  Simone Riva
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import numpy
import wave
import os
from dr14tmeter.audio_decoder import AudioDecoder


class AudioTrack:

    def __init__(self):
        self.Y = numpy.array([])
        self.Fs = 0
        self.channels = 0
        self.sample_width = 0 

    def time(self):
        return 1/self.Fs * self.Y.shape[0]

    def open( self , file_name ):

        self.Y = numpy.array([])
        self.Fs = 0
        self.channels = 0

        if not ( os.path.exists( file_name ) ) :
            return False

        de = AudioDecoder()

        res_f = de.read_track_new( file_name , self )

        return res_f 
  



# dr14_t.meter: compute the DR14 value of the given audiofiles
# Copyright (C) 2011  Simone Riva
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import argparse
from dr14tmeter.dr14_global import dr14_version


def parse_args():
    desc = "Compute the DR14 value of the audio files according to the algorithm " 
    desc =  desc + "described by the Pleasurize Music Foundation "
    desc =  desc + "Visit: http://www.dynamicrange.de/"

    parser = argparse.ArgumentParser( description=desc  )


    parser.add_argument("-1", "--disable_multithread",
        action="store_true",
        dest="disable_multithread",
        help="Disable the multi-Core mode")
    
    parser.add_argument("-r", "--recursive",
        action="store_true",
        dest="recursive",
        help="Scan recursively the subdirectories")

    parser.add_argument("-a", "--append",
        action="store_true",
        dest="append",
        help="Append all results in a single file; it should be used in couple with -r")

    parser.add_argument("-b", "--basic_table",
        action="store_true",
        dest="basic_table",
        help="Write the resulting tables in the basic format")

    parser.add_argument("-n", "--turn_off_out",
        action="store_true",
        dest="turn_off_out",
        help="do not writes the output files")

    parser.add_argument("-p", "--print_std_out",
        action="store_true",
        dest="print_std_out",
        help="writes the full result on the std_out")

    parser.add_argument("-o", "--outdir",
        action="store",
        dest="out_dir",
        type=str,
        help="Write the resultings files into the given directory")
    
    parser.add_argument("-t", "--tables",
        action="store",
        choices = 'htbwa',
        dest="out_tables",
        default="t" ,
        help="Select the output files to be written. h=html t=text b=bbcode w=mediawiki a=all_formats")

    parser.add_argument("-f", "--file",
        action='store_true',
        dest="scan_file",
        help="Compute the DR14 of a single file and exit")

    parser.add_argument("-d", "--dr_database",
        action="store_false",
        dest="dr_database",
        help="Output file compatible with the DR database at http:///www.dr.loudness-war.info" )

    parser.add_argument( "--hist" ,
        action="store_true",
        dest="histogram" ,
        help="Plot the histogram of dynamic of a single file and exit" )
    
    parser.add_argument( "--lev_hist" ,
        action="store_true",
        dest="lev_histogram" ,
        help="Plot the histogram of the saples levels of a single file and exit" )

    parser.add_argument( "--spectrogram" ,
        action="store_true",
        dest="spectrogram" ,
        help="Plot the spectrogram of a single file and exit (beta)" )
    
    parser.add_argument( "--plot_track" ,
        action="store_true",
        dest="plot_track" ,
        help="Plot the track of the given file and exit" )

    parser.add_argument( "--plot_track_dst" ,
        action="store_true",
        dest="plot_track_dst" ,
        help="Plot the track in a cool manner of the given file and exit" )

    parser.add_argument( "--compress" ,
        action="store" ,
        choices=[ "very_soft" , "v" , "soft" , "s" , "medium" , "m" , "hard" , "h" , "very_hard" , "vh" ] ,
        default=None ,
        dest="compress" ,
        help="Perform the dynamic compression on a single file and exit, the resulting track will be written in a temp directory" )

    parser.add_argument( "--dyn_vivacity" ,
        action="store_true" ,
        dest="dynamic_vivacity" ,
        help="Plot the graph of the dynamic vivacity of a single audio file and exit" )

    parser.add_argument("--files_list",
        action='store_true',
        dest="files_list",
        help="takes a files list and compute the DR of each file, if no files are specified, it reads from STDIN")

    parser.add_argument("--quiet",
        action="store_true",
        dest="quiet",
        help="Quiet mode")
    
    parser.add_argument("--version",
        action="store_true",
        dest="version",
        help="print the current version and exit")

    parser.add_argument(
        dest="path_name",
        nargs='?',
        default=None
        )
    
    return parser.parse_args()


# dr14_t.meter: compute the DR14 value of the given audiofiles
# Copyright (C) 2011 - 2012  Simone Riva
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import numpy
from dr14tmeter.audio_math import *

try:
    import matplotlib.pyplot as pyplot
    import matplotlib.mlab as mlab
except:
    ____foo = None
    

def compute_lev_hist ( Y , Fs , duration=None , bins=100 , plot=True , title=None ):
    
    s = Y.shape
    
    if len( Y.shape ) > 1 :
        ch = s[1]
    else :
        ch = 1
    
    Ym = numpy.sum( Y , 1 ) / float(ch)

    if plot :
        ( hist , bin_edges , patches ) = pyplot.hist(  Ym  , 100 , normed=True  )
        
        pyplot.grid(True)
        
        if title != None:
            hist_title = title
        else:
            hist_title = "Hystogram of levels"
        
        pyplot.title(r'%s'%hist_title)
        
        pyplot.show()
    else:
        ( hist , bin_edges ) = numpy.histogram( rms , bins=bins , normed=True )
        
    return ( hist , bin_edges )


# dr14_t.meter: compute the DR14 value of the given audiofiles
# Copyright (C) 2011  Simone Riva
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import time
import os
import sys
import tempfile
import subprocess
import re
import wave
import numpy


from io import StringIO

from dr14tmeter.out_messages import print_msg, dr14_log_info
from dr14tmeter.dr14_global import get_ffmpeg_cmd

#from _ftdi1 import NONE

# ffmpeg -i example.m4a -f wav pipe:1 > test.wav

class AudioFileReader:
    
    def __init__(self):
        
        self.__ffmpeg_cmd = get_ffmpeg_cmd()
        
        if sys.platform.startswith('win'):
            self.__cmd = ".\\decoder\\%s " % self.get_cmd()
        else:
            self.__cmd = "%s " % self.get_cmd()

    def get_cmd(self):
        pass

    def get_ffmpeg_cmd(self):
        return self.__ffmpeg_cmd

    def get_cmd_options( self , file_name , tmp_file ):
        pass

    def to_wav( self , file_name ):
        
        full_command = self.__cmd
        
        (head, file) = os.path.split( file_name )
        tmp_dir = tempfile.gettempdir()
        tmp_file = os.path.join( tmp_dir , file ) + ".wav"
        
        file_name = re.sub( "(\"|`|\$)" , r"\\\1" , file_name )
        tmp_file = re.sub( "(\"|`|\$)" , r"_xyz_" , tmp_file )
        
        full_command = full_command + " " + self.get_cmd_options( file_name , tmp_file )
        
        r = subprocess.call( full_command , shell=True  , stderr=subprocess.PIPE , stdout=subprocess.PIPE )
        
        if os.path.exists( tmp_file ) :
            return tmp_file
        else :
            return "" 
  

    def read_audio_file_new( self , file_name , target ):
        
        time_a = time.time()
                
        full_command = self.__cmd
        
        (head, file) = os.path.split( file_name )
        tmp_dir = tempfile.gettempdir()
        tmp_file = os.path.join( tmp_dir , file ) + ".wav"
        
        file_name = re.sub( "(\"|`|\$)" , r"\\\1" , file_name )
        tmp_file = re.sub( "(\"|`|\$)" , r"_xyz_" , tmp_file )
        
        full_command = full_command + " " + self.get_cmd_options( file_name , tmp_file )
        
        #print_msg( full_command )
        
        r = subprocess.call( full_command , shell=True  , stderr=subprocess.PIPE , stdout=subprocess.PIPE )
                
        #read_wav.read_wav( tmp_file )
        
        ret_f = self.read_wav( tmp_file , target )
        
        if os.path.exists( tmp_file ) :
            os.remove( tmp_file )
        else:
            print_msg( file_name + ": unsupported encoder" )
             
        time_b = time.time()
        dr14_log_info( "AudioFileReader.read_audio_file_new: Clock: %2.8f" % (time_b - time_a ) )
        
        return ret_f 


    def read_wav( self , file_name , target ):
    
        time_a = time.time()
    
        convert_8_bit =  numpy.float32( 2**8 + 1.0 )
        convert_16_bit = numpy.float32( 2**15 + 1.0 )
        convert_32_bit = numpy.float32( 2**31 + 1.0 )
                
        try:
            wave_read = wave.open( file_name , 'r' )
            target.channels = wave_read.getnchannels()
            target.Fs = wave_read.getframerate()
            target.sample_width = wave_read.getsampwidth()
            
            nframes = wave_read.getnframes()
            #print_msg( file_name + "!!!!!!!!!!!!: " + str(target.channels) + " " + str(target.sample_width ) + " " + str( target.Fs ) + " " + str( nframes ) )
            
            X = wave_read.readframes( wave_read.getnframes() )
                                    
            sample_type = "int%d" % ( target.sample_width * 8 )
        
            target.Y = numpy.fromstring(X, dtype=sample_type ).reshape( nframes , target.channels )
            
            wave_read.close()

            if sample_type == 'int16':
                target.Y = target.Y / ( convert_16_bit )
            elif sample_type == 'int32':
                target.Y = target.Y / ( convert_32_bit )
            else :
                target.Y = target.Y / ( convert_8_bit )
                
            #print_msg( "target.Y: " + str(target.Y.dtype) )
        except:
            self.__init__()
            print_msg ( "Unexpected error: %s" % str( sys.exc_info() ) )
            print_msg (  "\n - ERROR ! " )
            return False
     
        time_b = time.time()
        dr14_log_info( "AudioFileReader.read_wav: Clock: %2.8f" % (time_b - time_a ) )
        
        return True
    
    def get_generic_ffmpeg_options( self , file_name , tmp_file ):
        return  " -i \"%s\" -b 16 -ar 44100 -y \"%s\" -loglevel quiet " % ( file_name , tmp_file )



class Mp3FileReader( AudioFileReader ):
    def get_cmd(self):
        return "lame"
    
    def get_cmd_options(self , file_name , tmp_file ):
        return "--silent " + "--decode " + "\"" + file_name + "\"" + " \"%s\" " % tmp_file


class FlacFileReader( AudioFileReader ):
    def get_cmd(self):
        return self.get_ffmpeg_cmd()
    
    def get_cmd_options(self , file_name , tmp_file ):
        return self.get_generic_ffmpeg_options( file_name , tmp_file )
        #return " -s " + "-d " + "\"" + file_name + "\"" + " -o \"%s\" " % tmp_file


class Mp4FileReader( AudioFileReader ):
    def get_cmd(self):
        return self.get_ffmpeg_cmd()
    
    def get_cmd_options(self , file_name , tmp_file ):
        return  self.get_generic_ffmpeg_options( file_name , tmp_file )


class OggFileReader( AudioFileReader ):
    def get_cmd(self):
        return "oggdec"
    
    def get_cmd_options(self , file_name , tmp_file ):
        return  "--quiet " + "\"" + file_name + "\"" + " --output \"%s\"  " % tmp_file

class ApeFileReader( AudioFileReader ):
    def get_cmd(self):
        return self.get_ffmpeg_cmd()
    
    def get_cmd_options(self , file_name , tmp_file ):
        return  self.get_generic_ffmpeg_options( file_name , tmp_file )


class Ac3FileReader( AudioFileReader ):
    def get_cmd(self):
        return self.get_ffmpeg_cmd()

    def get_cmd_options(self , file_name , tmp_file ):
        return  self.get_generic_ffmpeg_options( file_name , tmp_file )


class WmaFileReader( AudioFileReader ):
    def get_cmd(self):
        return self.get_ffmpeg_cmd()
    
    def get_cmd_options(self , file_name , tmp_file ):
        return  self.get_generic_ffmpeg_options( file_name , tmp_file )


class WavFileReader( AudioFileReader ):
    def read_audio_file_new( self , file_name , target ):
        return self.read_wav( file_name , target )
    
    def get_cmd(self):
        return ""

    def get_cmd_options( self , file_name , tmp_file ):
        return ""




# dr14_t.meter: compute the DR14 value of the given audiofiles
# Copyright (C) 2011  Simone Riva
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import os
import shutil
import subprocess
import sys
import re


from distutils.core import setup
from dr14tmeter.dr14_global import dr14_version
from distutils.command.install import install


class dr14_install(install):
    user_options = install.user_options
 
    def run(self):
        
        install.run(self)
        
        man_dir = os.path.abspath("./man/")

        #print ( "root:   " + self.root )
        #print ( "prefix: " + self.prefix )

        prefix = re.sub( r'^/' , '' , self.prefix )

        output = subprocess.Popen([os.path.join(man_dir, "install.sh")],
                stdout=subprocess.PIPE,
                cwd=man_dir,
                env=dict({"PREFIX": os.path.join( self.root , prefix ) }, **dict(os.environ))).communicate()[0]
        print( output )




setup(name = "dr14_tmeter",
    version = "%s" % dr14_version() ,
    description = "Compute the DR14 value of the given audio files",
    author = "Simone Riva",
    author_email = "simone.rva [at] gmail.com",
    url = "http://simon-r.github.com/dr14_t.meter",
    packages = ['dr14tmeter'],
    scripts = ["dr14_tmeter"],
    long_description = "Compute the DR14 value of the given audio files according to the algorithm decribed by the Pleasurize Music Foundation" ,
    classifiers=[
        'Development Status :: %s Stable' % dr14_version() ,
        'Environment :: Console',
        'Intended Audience :: Users',
        'License :: OSI Approved :: GPL-3.0+',
        'Operating System :: Linux',
        'Programming Language :: Python',
        'Topic :: Multimedia'] ,
    cmdclass={"install": dr14_install }
) 



# dr14_t.meter: compute the DR14 value of the given audiofiles
# Copyright (C) 2011  Simone Riva
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import os
import logging
import codecs

message_file = sys.stderr
out_file = sys.stdout
err_file = sys.stderr
mode = "verbose"

logger = logging.getLogger('dr14log')

def init_log( lev=logging.DEBUG ):
    global logger
        
    logger = logging.getLogger('dr14log')
    logger.setLevel( logging.DEBUG )
    stream_h = logging.StreamHandler()
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    
    stream_h.setLevel( lev )
    stream_h.setFormatter( formatter )
    
    logger.addHandler( stream_h )

def dr14_log_debug( message ):
    global logger    
    logger.debug( message )

def dr14_log_info( message ):
    global logger    
    logger.info( message )

########

def print_msg( string ) :
    global message_file
    message_file.write( "%s\n" % string )
   
def print_err( string ):
    global err_file
    err_file.write( "Error: %s \n" % string )
    
def print_out( string ) :
    global out_file
    out_file.write( "%s\n" % string )

    
def set_verbose_msg() :
    global message_file
    global mode
    
    if mode == "verbose" :
        return 
    
    codecs.close( message_file )
    
    message_file = sys.stderr

    
def set_quiet_msg() :
    global message_file
    global mode
    
    if mode == "quiet" :
        return 
    
    message_file = codecs.open( os.devnull , "w" ,  encoding="utf-8")
    
    
    



# dr14_t.meter: compute the DR14 value of the given audiofiles
# Copyright (C) 2011 - 2012  Simone Riva
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import threading
import sys
import codecs
import tempfile

from dr14tmeter.compute_dr14 import compute_dr14
from dr14tmeter.compute_drv import compute_DRV
from dr14tmeter.compute_dr import *
from dr14tmeter.audio_track import *
from dr14tmeter.table import *
from dr14tmeter.dr_histogram import *
from dr14tmeter.lev_histogram import *
from dr14tmeter.spectrogram import *
from dr14tmeter.compressor import DynCompressor
from dr14tmeter.wav_write import wav_write
from dr14tmeter.read_metadata import RetirveMetadata
from dr14tmeter.audio_decoder import AudioDecoder
from dr14tmeter.duration import StructDuration
from dr14tmeter.write_dr import WriteDr, WriteDrExtended
from dr14tmeter.dynamic_vivacity import dynamic_vivacity
from dr14tmeter.plot_track import *
from dr14tmeter.plot_track_classic import *

import dr14tmeter.dr14_global as dr14

from dr14tmeter.out_messages import print_msg, print_out


class AudioAnalysis :
    
    def compute_track( self , file_name ) :
        self.at = AudioTrack()
        
        ( head,  file_n ) = os.path.split( file_name )
        
        if not self.at.open( file_name ):
            return False
        
        self.file_name = file_name
        
        self.meta_data = RetirveMetadata()
        self.meta_data.scan_file( self.file_name )
        
        self.duration = StructDuration()
        self.duration.set_samples( self.at.Y.shape[0] , self.at.Fs )
        
        self.virt_compute()
        
        return True
    
    def getDuration(self):
        return self.duration
    
    def getMetaData(self):
        return self.meta_data

    def getAudioTrack(self):
        return self.at
    
    def getFileName(self):
        return self.file_name
    
    def virt_compute(self):
        raise
    

class AudioDynVivacity( AudioAnalysis ):
    
    def virt_compute(self):
        (foo,fn) = os.path.split( self.getFileName() )
        
        title = self.getMetaData().get_value( fn , "title" )
        
        print_msg( "Track Title: %s " % title )
        
        at = self.getAudioTrack()
        dynamic_vivacity( at.Y , at.Fs )


class AudioDrHistogram( AudioAnalysis ):
    
    def virt_compute(self):
        
        (foo,fn) = os.path.split( self.getFileName() )
        
        title = self.getMetaData().get_value( fn , "title" )
        
        print_msg( "Track Title: %s " % title )
        
        at = self.getAudioTrack()
        compute_hist( at.Y , at.Fs , self.getDuration() , title=title )
        

class AudioLevelHistogram( AudioAnalysis ):
    
    def virt_compute(self):
        
        (foo,fn) = os.path.split( self.getFileName() )
        
        title = self.getMetaData().get_value( fn , "title" )
        
        print_msg( "Track Title: %s " % title )
        
        at = self.getAudioTrack()
        compute_lev_hist( at.Y , at.Fs , self.getDuration() , title=title )


class AudioSpectrogram( AudioAnalysis ):
    
    def virt_compute(self):
        
        (foo,fn) = os.path.split( self.getFileName() )
        
        title = self.getMetaData().get_value( fn , "title" )
        
        print_msg( "Track Title: %s " % title )
        
        at = self.getAudioTrack()
        
        spectrogram( at.Y , at.Fs )
    

class AudioPlotTrack( AudioAnalysis ):
    
    def virt_compute(self):
        
        (foo,fn) = os.path.split( self.getFileName() )
        title = self.getMetaData().get_value( fn , "title" )
        print_msg( "Track Title: %s " % title )
        
        at = self.getAudioTrack()
        plot_str = plot_track_classic( at.Y , at.Fs )
        plot_str.start()



class AudioPlotTrackDistribution( AudioAnalysis ):
    
    def virt_compute(self):
        
        (foo,fn) = os.path.split( self.getFileName() )
        title = self.getMetaData().get_value( fn , "title" )
        print_msg( "Track Title: %s " % title )
        
        at = self.getAudioTrack()
        plot_track( at.Y , at.Fs )


class AudioCompressor( AudioAnalysis ):
    
    def setCompressionModality( self , compression_modality ):
        self.compression_modality = compression_modality
    
    def virt_compute(self):
        (head, file_n) = os.path.split( self.getFileName() )
        
        comp = DynCompressor()
        comp.set_compression_modality( self.compression_modality )
        
        full_file = os.path.join( tempfile.gettempdir() , "%s%s.wav" % ( file_n , "-compressed-" ) )
        
        at = self.getAudioTrack()
        cY = comp.dyn_compressor( at.Y , at.Fs )
        
        wav_write( full_file , at.Fs , cY )
        print_msg( "The resulting compressed audiotrack has been written in: %s " % full_file )
        
    


# dr14_t.meter: compute the DR14 value of the given audiofiles
# Copyright (C) 2011 - 2012  Simone Riva
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import threading
import sys
import codecs
import tempfile
import multiprocessing as mp

from ctypes import Structure, c_double, c_wchar_p, c_int

from dr14tmeter.compute_dr14 import compute_dr14
from dr14tmeter.compute_drv import compute_DRV
from dr14tmeter.compute_dr import *
from dr14tmeter.audio_track import *
from dr14tmeter.table import *
from dr14tmeter.dr_histogram import *
from dr14tmeter.lev_histogram import *
from dr14tmeter.spectrogram import *
from dr14tmeter.compressor import DynCompressor
from dr14tmeter.wav_write import wav_write
from dr14tmeter.read_metadata import RetirveMetadata
from dr14tmeter.audio_decoder import AudioDecoder
from dr14tmeter.duration import StructDuration
from dr14tmeter.write_dr import WriteDr, WriteDrExtended

import dr14tmeter.dr14_global as dr14

from dr14tmeter.out_messages import print_msg, print_out, dr14_log_debug


class SharedDrResObj(Structure):
    _fields_ = [('file_name', c_wchar_p), ('dr14', c_double) , ('dB_peak' , c_double) , ( 'dB_rms' , c_double ) , ( 'duration' , c_double ) ]


class DynamicRangeMeter:   
    
    def __init__( self ):
        self.res_list = []
        self.dir_name = '' 
        self.dr14 = 0 
        self.meta_data = RetirveMetadata()        
        self.compute_dr = ComputeDR14()
    
    def scan_file( self , file_name):
        
        at = AudioTrack() 
        
        duration = StructDuration() 
        
        if at.open( file_name ):
            self.__compute_and_append( at , file_name )
            return 1
        else:
            return 0
        
    def scan_dir( self , dir_name ):
        
        if not os.path.isdir(dir_name) :
            return 0
        
        dir_list = sorted( os.listdir( dir_name ) )
        
        self.dir_name = dir_name 
        self.dr14 = 0 
        
        duration = StructDuration() 
        
        at = AudioTrack() 
        for file_name in dir_list:
            full_file = os.path.join( dir_name , file_name )
            
            #print_msg( full_file )
            if at.open( full_file ):
                self.__compute_and_append( at , file_name )

        self.meta_data.scan_dir( dir_name )
        if len( self.res_list ) > 0:
            self.dr14 = int( round( self.dr14 / len( self.res_list ) ) )
            return len( self.res_list )
        else:
            return 0
 

    def __compute_and_append( self , at , file_name ):
        
        duration = StructDuration()
        
        #( dr14, dB_peak, dB_rms ) = self.compute_dr.compute( at.Y , at.Fs )
        ( dr14, dB_peak, dB_rms ) = compute_dr14( at.Y , at.Fs , duration )
        
        self.dr14 = self.dr14 + dr14
        
        res = { 'file_name': file_name , 'dr14': dr14 , 'dB_peak': dB_peak , 'dB_rms': dB_rms , 'duration':duration.to_str() }
        self.res_list.append(res)
        
        print_msg( file_name + ": \t DR " + str( int(dr14) ) )
  


    def fwrite_dr( self , file_name , tm , ext_table=False , std_out=False , append=False , dr_database=True ):
        
        if ext_table :
            wr = WriteDrExtended()
        else :
            wr = WriteDr()
        
        wr.set_dr_database( dr_database )
        
        self.table_txt = wr.write_dr( self , tm )        
        
        if std_out:
            print_out( self.table_txt )
            return 
        
        if append :
            file_mode = "a"
        else :
            file_mode = "w"
        
        try:
            out_file = codecs.open( file_name , file_mode , "utf-8-sig" )
        except:
            print_msg ( "File opening error [%s] :" % file_name , sys.exc_info()[0] )
            return False
        
        out_file.write( self.table_txt )
        out_file.close()
        return True

    def scan_mp( self , dir_name="" , thread_cnt=2 , files_list=[] ):
        
        if sys.version_info[0] == 2 :
            dir_name = dir_name.decode('utf-8')
        
        self.dr14 = 0
                
        if files_list == [] :
            if not os.path.isdir(dir_name) :
                return 0
            dir_list = sorted( os.listdir( dir_name ) )
            self.dir_name = dir_name
            files_list = None
        else:
            dir_list = sorted( files_list )
            
        ad = AudioDecoder()
        
        jobs = []
        for file_name in dir_list:
            ( fn , ext ) = os.path.splitext( file_name )
            if ext in ad.formats:
                jobs.append( file_name )
        
        
        res_array=[SharedDrResObj() for i in range( len(jobs) )]
        
        for i in range( len(jobs) ) :
            res_array[i].file_name = jobs[i]
            res_array[i].dr14 = dr14.min_dr()
        
        lock_j = mp.Lock()
        lock_res_list = mp.Lock()
        
        threads = [1 for i in range(thread_cnt)]
        
        #job_free = [0]
        job_free = mp.Value( 'i' , 0 )
        res_array_sh = mp.Array( SharedDrResObj , res_array )
        
        for t in range( thread_cnt ):
            threads[t] = mp.Process( target=self.run_mp , args=( dir_name , lock_j , lock_res_list , job_free , res_array_sh ) )
            
        for t in range( thread_cnt ):
            threads[t].start() 
        
        for t in range( thread_cnt ):
            threads[t].join()
            
        succ = 0
        
        #empty_res = { 'file_name': '' , 'dr14': dr14.min_dr() , 'dB_peak': -100 , 'dB_rms': -100 , 'duration':"0:0" }
        self.res_list = [] # [empty_res for i in range( len(jobs) )]
        
        #i = 0
        
        dur = StructDuration()
        
        for res in res_array_sh:
            self.res_list.append( { 'file_name':   res.file_name ,
                                 'dr14':        res.dr14 ,
                                 'dB_peak':     res.dB_peak ,
                                 'dB_rms':      res.dB_rms ,
                                 'duration':    dur.float_to_str( res.duration ) } )
            
        #    i = i + 1
        
        for d in self.res_list:
            if d['dr14'] > dr14.min_dr():
                self.dr14 = self.dr14 + d['dr14']
                succ = succ + 1
                
        self.meta_data.scan_dir( dir_name , files_list )
        
        if len( self.res_list ) > 0 and succ > 0 :
            self.dr14 = int( round( self.dr14 / succ ) )
            return succ
        else:
            return 0
  
    
    def run_mp( self , dir_name , lock_j , lock_res_list , job_free , res_array_sh ):
        
        at = AudioTrack() 
        duration = StructDuration()
        
        #print_msg("start .... ")
        
        while True:
            
            #Aquire the next free job
            lock_j.acquire()
            
            if job_free.value >= len(res_array_sh):
                lock_j.release()
                return
            
            curr_job =  job_free.value
            file_name = res_array_sh[curr_job].file_name
            job_free.value = job_free.value + 1
            
            lock_j.release()
            
            full_file = os.path.join( dir_name , file_name )
            #print ( full_file )
            
            if at.open( full_file ):
                ( dr14, dB_peak, dB_rms ) = compute_dr14( at.Y , at.Fs , duration )
                lock_res_list.acquire()
                print_msg( file_name + ": \t DR " + str( int(dr14) ) )
                
                #res_list[curr_job] = { 'file_name': file_name , 'dr14': dr14 , 'dB_peak': dB_peak , 'dB_rms': dB_rms , 'duration':duration.to_str() }
                res_array_sh[curr_job].dr14 = dr14
                res_array_sh[curr_job].dB_peak = dB_peak
                res_array_sh[curr_job].dB_rms = dB_rms
                res_array_sh[curr_job].duration = duration.to_float() 
                
                lock_res_list.release()
            else:
                print_msg( "- fail - " + full_file )
    


   


# dr14_t.meter: compute the DR14 value of the given audiofiles
# Copyright (C) 2011  Simone Riva
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import numpy
from dr14tmeter.audio_math import *

import wave
    
def wav_write( filename , Fs , Y ):
    
    amplitude = 2.0**16 - 1.0
    
    wav_file = wave.open(filename, "w")
    
    s = Y.shape
    
    if len( Y.shape ) > 1 :
        nchannels = s[1]
    else :
        nchannels = 1
        
    sampwidth = 2
    framerate = int(Fs)
    
    nframes = s[0]
    comptype = "NONE"
    compname = "no comp"
    
    wav_file.setparams(( nchannels , sampwidth , framerate , nframes , comptype , compname ))
    
    Y_s = numpy.int16( (amplitude/2.0) * Y )
    Y_s = Y_s.tostring() ;
    
    wav_file.writeframes( Y_s )
        
    wav_file.close()
    